import { ɵɵdefineInjectable, Injectable } from '@angular/core';
import { __awaiter } from 'tslib';

class BaseFactory {
    constructor() {
        this._mtmAuto = null;
    }
    setAuto(service) {
        this._mtmAuto = service;
        return this;
    }
    getAuto() {
        return this._mtmAuto;
    }
}

class BaseTool {
    constructor() {
        this._mtmAuto = null;
    }
    setAuto(service) {
        this._mtmAuto = service;
        return this;
    }
    getAuto() {
        return this._mtmAuto;
    }
}

class v4 extends BaseTool {
    //USE: this.getAuto().get("utilities").getGuids().getV4().__METHOD__();
    get(inclBraces = true) {
        //replace the s4 function with random string from 
        function s4() {
            return Math.floor((1 + Math.random()) * 0x10000).toString(16).substring(1);
        }
        var guid = (s4() + s4() + '-' + s4() + '-' + s4() + '-' + s4() + '-' + s4() + s4() + s4()).toUpperCase();
        if (inclBraces === true) {
            return "[" + guid + "]";
        }
        else {
            return guid;
        }
    }
    isValid(input, inclBraces = true, throwOnErr = false) {
        var isValid = true;
        if (typeof input === "string") {
            if (inclBraces === true) {
                if (input.length !== 38) {
                    isValid = false;
                }
                else if (input.slice(input.length - 1) !== "]") {
                    isValid = false;
                }
                else if (input.slice(0, 1) !== "[") {
                    isValid = false;
                }
                else {
                    input = input.substring(1, input.length - 1);
                }
            }
            else if (input.length !== 36) {
                isValid = false;
            }
            if (isValid === true) {
                var parts = input.split("-");
                if (parts.length === 5) {
                    if (parts[0].length === 8
                        || parts[1].length === 4
                        || parts[2].length === 4
                        || parts[3].length === 4
                        || parts[4].length === 12) {
                        const regex = /^[A-F0-9]+$/;
                        if (parts.join("").match(regex) === null) {
                            isValid = false;
                        }
                    }
                    else {
                        isValid = false;
                    }
                }
                else {
                    isValid = false;
                }
            }
        }
        else {
            isValid = false;
        }
        if (isValid === true) {
            return true;
        }
        else if (throwOnErr === true) {
            throw this.getAuto().newException("Invalid v4 guid", 7869);
        }
        else {
            return false;
        }
    }
}

class Guids extends BaseFactory {
    constructor() {
        super(...arguments);
        this._v4 = null;
    }
    getV4() {
        if (this._v4 === null) {
            this._v4 = new v4();
            this._v4.setAuto(this.getAuto());
        }
        return this._v4;
    }
}

class Strings extends BaseTool {
    //USE: this.getAuto().get("utilities").getArrays().getStrings().__METHOD__();
    fromBuffer(buf) {
        return String.fromCharCode.apply(null, new Uint8Array(buf));
    }
}

class Objects extends BaseTool {
    //USE: this.getAuto().get("utilities").getArrays().getObjects().__METHOD__();
    isValid(input, throwOnErr = false) {
        if (Array.isArray(input) === true) {
            return true;
        }
        else if (throwOnErr === true) {
            throw this.getAuto().newException("Input is not an array object", 7885);
        }
        else {
            return false;
        }
    }
    isLength(input, len, throwOnErr = false) {
        this.isValid(input, true);
        if (input.length === len) {
            return true;
        }
        else if (throwOnErr === true) {
            throw this.getAuto().newException("Input does not have the correct length", 7880);
        }
        else {
            return false;
        }
    }
    isMaxLength(input, len, throwOnErr = false) {
        this.isValid(input, true);
        if (input.length <= len) {
            return true;
        }
        else if (throwOnErr === true) {
            throw this.getAuto().newException("Input exceeds the max length", 7886);
        }
        else {
            return false;
        }
    }
    isMinLength(input, len, throwOnErr = false) {
        this.isValid(input, true);
        if (input.length >= len) {
            return true;
        }
        else if (throwOnErr === true) {
            throw this.getAuto().newException("Input does not meet the minimum length", 7887);
        }
        else {
            return false;
        }
    }
}

class Arrays extends BaseFactory {
    constructor() {
        super(...arguments);
        this._strTool = null;
        this._objTool = null;
    }
    getStrings() {
        if (this._strTool === null) {
            this._strTool = new Strings();
            this._strTool.setAuto(this.getAuto());
        }
        return this._strTool;
    }
    getObjects() {
        if (this._objTool === null) {
            this._objTool = new Objects();
            this._objTool.setAuto(this.getAuto());
        }
        return this._objTool;
    }
}

class Random extends BaseTool {
    getByRegex(len, regex = "A-Za-z0-9\\#\\-\\+\\:\\;\\.\\,\\!") {
        try {
            var str = "";
            var chars = this.getAuto().get("utilities").getStrings().getCharacterSets().getAsciiPrintable();
            var uChars = chars.replace(new RegExp("[^" + regex + "]", "g"), "");
            var charLen = uChars.length;
        }
        catch (e) {
            throw this.getAuto().newException("Failed to limit character set exception: " + e);
        }
        if (charLen > 0) {
            for (var i = 0; i < len; i++) {
                str += uChars.charAt(Math.floor(Math.random() * charLen));
            }
            return str;
        }
        else {
            throw this.getAuto().newException("Invalid regex");
        }
    }
}

class Urls extends BaseTool {
    //USE: this.getAuto().get("utilities").getStrings().getUrl().__METHOD__();
    get() {
        return this.parse(window.location.href);
    }
    set(url) {
        window.location.href = url;
        return this;
    }
    parse(url) {
        var items = url.split('/').filter(function (el) {
            return el != "";
        });
        //if there is no / between the path and query we need to break it up
        var lItem = items[(items.length - 1)];
        const qBreak = lItem.indexOf("?");
        if (qBreak > 0) {
            items[(items.length - 1)] = lItem.substring(0, qBreak);
            items.push(lItem.substring(qBreak));
        }
        var scheme = items[0].toLowerCase().replace(/\:$/, "");
        var host = null;
        var port = null;
        var path = null;
        var query = {};
        var ps = items[1].split(":");
        host = ps[0];
        if (ps.length == 2) {
            port = parseInt(ps[1]);
        }
        else if (scheme == "http") {
            port = 80;
        }
        else if (scheme == "https") {
            port = 443;
        }
        if (items.length > 2) {
            for (var x = 2; x < items.length; x++) {
                var item = items[x];
                if (item.indexOf("?") === 0) {
                    ps = item.replace(/^\?/, "").split("&");
                    for (var y = 0; y < ps.length; y++) {
                        var qItem = ps[y].split("=");
                        query[qItem[0]] = qItem[1];
                    }
                }
                else {
                    path = item;
                }
            }
        }
        return { scheme: scheme, host: host, port: port, path: path, query: query };
    }
    getQueryValue(url, attr, doThrow = false) {
        var pObj = this.parse(url);
        if (typeof pObj.query[attr] != "undefined") {
            return pObj.query[attr];
        }
        else if (doThrow === true) {
            throw this.getAuto().newException("Atribute: '" + attr + "' does not exist");
        }
        else {
            return null;
        }
    }
}

class Arrays$1 extends BaseTool {
    toBuffer(str) {
        const buf = new ArrayBuffer(str.length);
        const buf2 = new Uint8Array(buf);
        for (let x = 0, strLen = str.length; x < strLen; x++) {
            buf2[x] = str.charCodeAt(x);
        }
        return buf;
    }
}

class Passwords extends BaseTool {
    //Use:
    //var any	= something.getAuto().get("utilities").getStrings().getPasswords().__METHOD__();
    getHumanV1(len = 9) {
        //prebuilt to fulfill the V1 requirements but excludes characters that can appear ambigous
        if (len < 9) {
            throw this.getAuto().newException("V1 passwords must be at least 9 character");
        }
        var regEx = "A-Ha-hJ-Nm-nP-Zp-z1-9\\#\\+\\-\\_\\@\\&\\%\\$\\*";
        var toolObj = this.getAuto().get("utilities").getStrings().getRandom();
        while (true) {
            var password = toolObj.getByRegex(len, regEx);
            if (this.validV1(password, false) === true) {
                return password;
            }
        }
    }
    getV1(len = 9) {
        //prebuilt to fulfill the V1 requirements
        if (len < 9) {
            throw this.getAuto().newException("V1 passwords must be at least 9 character");
        }
        var regEx = "A-Za-z0-9\\!\\#\\$\\%\\&\\'\\(\\)\\*\\+\\,\\-\\.\\/\\:\\;\\<\\=\\>\\?\\@\\{\\|\\}\\~\\[\\]\\^";
        var toolObj = this.getAuto().get("utilities").getStrings().getRandom();
        while (true) {
            var password = toolObj.getByRegex(len, regEx);
            if (this.validV1(password, false) === true) {
                return password;
            }
        }
    }
    validV1(input, throwOnErr = false) {
        //prebuilt for easy checking
        //ascii cannot start or end with a space
        //must have one upper, lower a symbol and be at least 9 char
        if (typeof input !== "string") {
            throw this.getAuto().newException("Invalid input, only accepts strings");
        }
        var isValid = true;
        var errMsg = "";
        if (isValid === true && input.length !== input.trim().length) {
            isValid = false;
            errMsg = "Cannot start or end with a space";
        }
        if (isValid === true && input.length < 9) {
            isValid = false;
            errMsg = "Password must be at least 9 character";
        }
        if (isValid === true) {
            isValid = this.asciiIncl(input, true, true, true, true, throwOnErr);
        }
        if (isValid === true) {
            return true;
        }
        else if (throwOnErr === true) {
            throw this.getAuto().newException("Password is invalid. " + errMsg);
        }
        else {
            return false;
        }
    }
    asciiIncl(input, upper = true, lower = true, number = true, symbol = true, throwOnErr = false) {
        if (typeof input !== "string") {
            throw this.getAuto().newException("Invalid input, only accepts strings");
        }
        var isValid = true;
        var errMsg = "";
        var regTool = this.getAuto().get("utilities").getExpressions().getRegEx();
        var charTool = this.getAuto().get("utilities").getStrings().getCharacterSets();
        if (isValid === true && upper === true) {
            const regex = RegExp("[" + regTool.makeSafe(charTool.getAsciiUpper()) + "\]+");
            if (input.match(regex) === null) {
                isValid = false;
                errMsg = "Must contain atleast one uppercase letter";
            }
        }
        if (isValid === true && lower === true) {
            const regex = RegExp("[" + regTool.makeSafe(charTool.getAsciiLower()) + "\]+");
            if (input.match(regex) === null) {
                isValid = false;
                errMsg = "Must contain atleast one lowercase letter";
            }
        }
        if (isValid === true && number === true) {
            const regex = RegExp("[" + regTool.makeSafe(charTool.getDecimals()) + "\]+");
            if (input.match(regex) === null) {
                isValid = false;
                errMsg = "Must contain atleast one digit";
            }
        }
        if (isValid === true && symbol === true) {
            //we dont want space to count as a symbol
            const regex = RegExp("[" + regTool.makeSafe(charTool.getAsciiSymbols().replace(" ", "")) + "\]+");
            if (input.match(regex) === null) {
                isValid = false;
                errMsg = "Must contain atleast one symbol";
            }
        }
        if (isValid === true) {
            return true;
        }
        else if (throwOnErr === true) {
            throw this.getAuto().newException("Password is invalid. " + errMsg);
        }
        else {
            return false;
        }
    }
}

class Emails extends BaseTool {
    //USE: this.getAuto().get("utilities").getStrings().getEmail().__METHOD__();
    isValid(input, throwOnErr = false) {
        if (typeof input === "string") {
            //src: https://emailregex.com/
            const regEx = RegExp(/^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/);
            if (regEx.test(input) === true) {
                return true;
            }
        }
        if (throwOnErr === true) {
            throw this.getAuto().newException("Invalid email address", 7866);
        }
        else {
            return false;
        }
    }
}

class String$1 extends BaseTool {
    //USE: this.getAuto().get("utilities").getStrings().getString().__METHOD__();
    isValid(input, throwOnErr = false) {
        if (typeof input === "string") {
            return true;
        }
        else if (throwOnErr === true) {
            throw this.getAuto().newException("Input is not a string", 7873);
        }
        else {
            return false;
        }
    }
    isLength(input, len, throwOnErr = false) {
        this.getAuto().get("utilities").getNumbers().getIntegers().isValid(len, true);
        if (this.isValid(input, throwOnErr) === true) {
            if (input.length === len) {
                return true;
            }
            else if (throwOnErr === true) {
                throw this.getAuto().newException("Input string has incorrect length", 7874);
            }
        }
        return false;
    }
    isMaxLength(input, len, throwOnErr = false) {
        this.getAuto().get("utilities").getNumbers().getIntegers().isValid(len, true);
        if (this.isValid(input, throwOnErr) === true) {
            if (input.length <= len) {
                return true;
            }
            else if (throwOnErr === true) {
                throw this.getAuto().newException("Input string has excessive length", 7875);
            }
        }
        return false;
    }
    isMinLength(input, len, throwOnErr = false) {
        this.getAuto().get("utilities").getNumbers().getIntegers().isValid(len, true);
        if (this.isValid(input, throwOnErr) === true) {
            if (input.length >= len) {
                return true;
            }
            else if (throwOnErr === true) {
                throw this.getAuto().newException("Input string has insuficient length", 7876);
            }
        }
        return false;
    }
}

class MacAddresses extends BaseTool {
    //USE: this.getAuto().get("utilities").getStrings().getMacAddress().__METHOD__();
    isValid(input, throwOnErr = false) {
        if (typeof input === "string") {
            const regEx = RegExp(/^([a-fA-F0-9]{12})$/);
            if (regEx.test(input) === true) {
                return true;
            }
        }
        if (throwOnErr === true) {
            throw this.getAuto().newException("Invalid mac address", 7871);
        }
        else {
            return false;
        }
    }
}

class Ssid extends BaseTool {
    //USE: this.getAuto().get("utilities").getStrings().getSsid().__METHOD__();
    isValid(input, throwOnErr = false) {
        if (typeof input === "string") {
            const regEx = RegExp(/^([.]{1,32})$/); //a 0 bytes ssid is a wildcard
            if (regEx.test(input) === true) {
                return true;
            }
        }
        if (throwOnErr === true) {
            throw this.getAuto().newException("Invalid ssid", 7879);
        }
        else {
            return false;
        }
    }
}

class CharacterSets extends BaseTool {
    //Use:
    //var chars	= something.getAuto().get("utilities").getStrings().getCharacterSets().__METHOD__();
    getDecimals() {
        return "0123456789";
    }
    getAsciiSymbols() {
        return "!\"#$%&'()*+,-./:;<=>?@{|}~[\\]^ _`";
    }
    getAsciiLower() {
        return "abcdefghijklmnopqrstuvwxyz";
    }
    getAsciiUpper() {
        return "ABCDEFGHIJKLMNOPQRSTUVWXYZ";
    }
    getAsciiPrintable() {
        return this.getAsciiLower() + this.getAsciiUpper() + this.getDecimals() + this.getAsciiSymbols();
    }
}

class Strings$1 extends BaseFactory {
    constructor() {
        super(...arguments);
        this._randomTool = null;
        this._urlTool = null;
        this._arrTool = null;
        this._pwTool = null;
        this._emailTool = null;
        this._macTool = null;
        this._csTool = null;
        this._strTool = null;
        this._ssidTool = null;
    }
    getRandom() {
        if (this._randomTool === null) {
            this._randomTool = new Random();
            this._randomTool.setAuto(this.getAuto());
        }
        return this._randomTool;
    }
    getUrl() {
        if (this._urlTool === null) {
            this._urlTool = new Urls();
            this._urlTool.setAuto(this.getAuto());
        }
        return this._urlTool;
    }
    getString() {
        if (this._strTool === null) {
            this._strTool = new String$1();
            this._strTool.setAuto(this.getAuto());
        }
        return this._strTool;
    }
    getArrays() {
        //What are you doing here?
        if (this._arrTool === null) {
            this._arrTool = new Arrays$1();
            this._arrTool.setAuto(this.getAuto());
        }
        return this._arrTool;
    }
    getPasswords() {
        if (this._pwTool === null) {
            this._pwTool = new Passwords();
            this._pwTool.setAuto(this.getAuto());
        }
        return this._pwTool;
    }
    getEmail() {
        if (this._emailTool === null) {
            this._emailTool = new Emails();
            this._emailTool.setAuto(this.getAuto());
        }
        return this._emailTool;
    }
    getMacAddress() {
        if (this._macTool === null) {
            this._macTool = new MacAddresses();
            this._macTool.setAuto(this.getAuto());
        }
        return this._macTool;
    }
    getSsid() {
        if (this._ssidTool === null) {
            this._ssidTool = new Ssid();
            this._ssidTool.setAuto(this.getAuto());
        }
        return this._ssidTool;
    }
    getCharacterSets() {
        if (this._csTool === null) {
            this._csTool = new CharacterSets();
            this._csTool.setAuto(this.getAuto());
        }
        return this._csTool;
    }
}

class Integers extends BaseTool {
    //USE: this.getAuto().get("utilities").getNumbers().getIntegers().__METHOD__();
    isValid(input, throwOnErr = false) {
        if (typeof input === "number") {
            if (Number.isInteger(input) === true) {
                return true;
            }
        }
        if (throwOnErr === true) {
            throw this.getAuto().newException("Invalid integer", 7872);
        }
        else {
            return false;
        }
    }
    isStringInteger(input, throwOnErr = false) {
        //is the input an integer cast as a string?
        if (typeof input === "string") {
            const regEx = RegExp(/^\d+$/);
            if (regEx.test(input) === true) {
                return true;
            }
        }
        if (throwOnErr === true) {
            throw this.getAuto().newException("Input is not an integer typed as a string", 7877);
        }
        else {
            return false;
        }
    }
}

class Numbers extends BaseFactory {
    constructor() {
        super(...arguments);
        this._intTool = null;
    }
    getIntegers() {
        if (this._intTool === null) {
            this._intTool = new Integers();
            this._intTool.setAuto(this.getAuto());
        }
        return this._intTool;
    }
}

class RegExs extends BaseTool {
    //Use:
    //var any	= something.getAuto().get("utilities").getExpressions().getRegEx().__METHOD__();
    makeSafe(str) {
        if (typeof str !== "string") {
            throw this.getAuto().newException("Invalid input, only accepts strings");
        }
        return str.replace(/[.*+?^${}()|[\]\\]/g, '\\$&');
    }
}

class Expressions extends BaseFactory {
    constructor() {
        super(...arguments);
        this._regExs = null;
    }
    getRegEx() {
        if (this._regExs === null) {
            this._regExs = new RegExs();
            this._regExs.setAuto(this.getAuto());
        }
        return this._regExs;
    }
}

class Strings$2 extends BaseTool {
    constructor() {
        super(...arguments);
        this._trackers = {};
    }
    toStr(obj, id = null, depDelim = null) {
        var id;
        return __awaiter(this, void 0, void 0, function* () {
            //remove circular dependencies so any object can be stringified
            if (typeof obj !== "object") {
                throw this.getAuto().newException("Invalid input");
            }
            var isinit = false;
            if (id === null) {
                isinit = true;
                id = this.getAuto().get("utilities").getStrings().getRandom().getByRegex(12);
                this._trackers[id] = { "objs": [] };
            }
            try {
                var isArr = null;
                var eObj = null;
                var rObj = null;
                if (Array.isArray(obj) === true) {
                    isArr = true;
                    rObj = [];
                }
                else {
                    isArr = false;
                    rObj = {};
                }
                for (let key in obj) {
                    var value = obj[key];
                    if (value instanceof Node) {
                        //NodeObject
                    }
                    else if (value instanceof Window) {
                        //WindowObject
                    }
                    else if (typeof value === "object") {
                        var found = false;
                        for (let eId in this._trackers[id]["objs"]) {
                            if (this._trackers[id]["objs"][eId] === value) {
                                found = true;
                            }
                        }
                        if (found === false) {
                            this._trackers[id]["objs"].push(value);
                            if (isArr === false) {
                                rObj[key] = yield this.toStr(value, id, depDelim);
                            }
                            else {
                                rObj.push(yield this.toStr(value, id, depDelim));
                            }
                        }
                        else if (depDelim !== null) {
                            if (isArr === false) {
                                rObj[key] = depDelim;
                            }
                            else {
                                rObj.push(depDelim);
                            }
                        }
                    }
                    else {
                        if (isArr === false) {
                            rObj[key] = value;
                        }
                        else {
                            rObj.push(value);
                        }
                    }
                }
                if (isinit === true) {
                    delete this._trackers[id];
                    return JSON.stringify(rObj);
                }
                else {
                    return rObj;
                }
            }
            catch (errObj) {
                if (isinit === true && typeof this._trackers[id] !== "undefined") {
                    delete this._trackers[id];
                }
                throw errObj;
            }
        });
    }
}

class Types extends BaseTool {
    //USE: this.getAuto().get("utilities").getObjects().getTypes().__METHOD__();
    isValid(input, throwOnErr = false) {
        if (typeof input === "object" && input !== null) {
            return true;
        }
        else if (throwOnErr === true) {
            throw this.getAuto().newException("Input is not an object", 7887);
        }
        else {
            return false;
        }
    }
}

class Objects$1 extends BaseFactory {
    constructor() {
        super(...arguments);
        this._strTool = null;
        this._typesTool = null;
    }
    getStrings() {
        if (this._strTool === null) {
            this._strTool = new Strings$2();
            this._strTool.setAuto(this.getAuto());
        }
        return this._strTool;
    }
    getTypes() {
        if (this._typesTool === null) {
            this._typesTool = new Types();
            this._typesTool.setAuto(this.getAuto());
        }
        return this._typesTool;
    }
}

class Promises extends BaseTool {
    getInline() {
        //allows creation of a promise and continue work outside the promise
        //helpful in async functions
        //you can return the promise to the end user
        //do some work and when ready place the return value / data 
        //piObj["data"] = {"someThing": "anotherThing"}
        //then either piObj["resolve"]() or piObj["reject"]()
        //you can also just do: piObj["resolve"](data) or piObj["reject"](errObj)
        var piObj = { "promise": null, "resolve": null, "reject": null, data: null };
        piObj["promise"] = new Promise((resolve, reject) => {
            piObj["resolve"] = function (data = null) {
                if (piObj["data"] === null) {
                    resolve(data);
                }
                else {
                    resolve(piObj["data"]);
                }
                ;
            };
            piObj["reject"] = function (data = null) {
                if (piObj["data"] === null) {
                    reject(data);
                }
                else {
                    reject(piObj["data"]);
                }
                ;
            };
        });
        return piObj;
    }
}

class Async extends BaseFactory {
    constructor() {
        super(...arguments);
        this._promiseTool = null;
    }
    getPromises() {
        if (this._promiseTool === null) {
            this._promiseTool = new Promises();
            this._promiseTool.setAuto(this.getAuto());
        }
        return this._promiseTool;
    }
}

class SHA extends BaseTool {
    //USE: this.getAuto().get("utilities").getHashing().getSHA().__METHOD__();
    get256(input) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof crypto.subtle === "undefined") {
                throw this.getAuto().newException("Browser crypto is only available in a secure context (https)");
            }
            const dataBuff = this.getAuto().get("utilities").getStrings().getArrays().toBuffer(input);
            const digest = yield crypto.subtle.digest("SHA-256", dataBuff);
            return Array.from(new Uint8Array(digest)).map(b => b.toString(16).padStart(2, "0")).join("");
        });
    }
    is256(input, base = 16, throwOnErr = false) {
        var isValid = true;
        if (typeof input === "string") {
            if (base === 16) {
                if (input.length !== 64) {
                    isValid = false;
                }
                else {
                    const regex = /^[a-f0-9]+$/;
                    if (input.match(regex) === null) {
                        isValid = false;
                    }
                }
            }
            else {
                throw this.getAuto().newException("Not handled for base: " + base);
            }
        }
        else {
            isValid = false;
        }
        if (isValid === true) {
            return true;
        }
        else if (throwOnErr === true) {
            throw this.getAuto().newException("Invalid sha256 hash", 7870);
        }
        else {
            return false;
        }
    }
}

class Hashing extends BaseFactory {
    constructor() {
        super(...arguments);
        this._shaTool = null;
    }
    getSHA() {
        if (this._shaTool === null) {
            this._shaTool = new SHA();
            this._shaTool.setAuto(this.getAuto());
        }
        return this._shaTool;
    }
}

class Ports extends BaseTool {
    //Use: this.getAuto().get("utilities").getNetwork().getPorts().__METHOD__();
    isSecureOpen(host, port, timeout = 500) {
        //make async instead
        var pObj = this.getAuto().get("utilities").getAsync().getPromises().getInline();
        const timeInt = setTimeout(() => {
            if (pObj["data"] === null) {
                pObj["data"] = false;
                pObj["resolve"]();
            }
        }, timeout);
        var url = "https://" + host + ":" + port;
        fetch(url, { method: "GET", mode: "no-cors", cache: "no-cache", redirect: "follow" }).then((resp) => {
            clearTimeout(timeInt);
            if (pObj["data"] === null) {
                pObj["data"] = true;
                pObj["resolve"]();
            }
        }, (errObj) => {
            clearTimeout(timeInt);
            if (pObj["data"] === null) {
                pObj["data"] = false;
                pObj["resolve"]();
            }
        });
        return pObj.promise;
    }
}

class IPs extends BaseTool {
    //Use: this.getAuto().get("utilities").getNetwork().getIPs().__METHOD__();
    isV4(input, throwOnErr = false) {
        if (typeof input === "string") {
            const regEx = RegExp(/^([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})$/);
            if (regEx.test(input) === true) {
                let octets = input.match(regEx);
                if (parseInt(octets[1]) < 256 && parseInt(octets[2]) < 256 && parseInt(octets[3]) < 256 && parseInt(octets[4]) < 256) {
                    return true;
                }
            }
        }
        if (throwOnErr === true) {
            throw this.getAuto().newException("Input is not an ipv4 address", 7878);
        }
        else {
            return false;
        }
    }
}

class Networks extends BaseFactory {
    constructor() {
        super(...arguments);
        this._ports = null;
        this._ips = null;
    }
    getPorts() {
        if (this._ports === null) {
            this._ports = new Ports();
            this._ports.setAuto(this.getAuto());
        }
        return this._ports;
    }
    getIPs() {
        if (this._ips === null) {
            this._ips = new IPs();
            this._ips.setAuto(this.getAuto());
        }
        return this._ips;
    }
}

class Current extends BaseTool {
    getMicroEpoch(float = true) {
        var curTime = new Date();
        var mEpoch = curTime.getTime() / 1000;
        if (float === true) {
            //all numbers in JS are stored as double precision floating point
            return mEpoch;
        }
        else {
            //fixed length, need to account for no .
            return mEpoch.toString() + "0".repeat((17 - mEpoch.toString().length));
        }
    }
    getEpoch() {
        var curTime = new Date();
        return Math.floor(curTime.getTime() / 1000);
    }
    getFormatted(format = "") {
        return this.getAuto().get("utilities").getTime().getFormatter().fromEpoch(this.getEpoch(), format);
    }
}

class Formatter extends BaseTool {
    getObjFromEpoch(epoch) {
        var int = parseInt(epoch);
        var dateObj = new Date(0);
        dateObj.setUTCSeconds(int);
        return dateObj;
    }
    fromEpoch(epoch, format = "") {
        var pObj = this.getParts(epoch);
        format = format.replace("H", pObj["H"]);
        format = format.replace("h", pObj["h"]);
        format = format.replace("i", pObj["i"]);
        format = format.replace("A", pObj["A"]);
        format = format.replace("T", pObj["T"]);
        format = format.replace("m", pObj["m"]);
        format = format.replace("Y", pObj["Y"]);
        format = format.replace("n", pObj["n"]);
        format = format.replace("d", pObj["d"]);
        format = format.replace("i", pObj["i"]);
        format = format.replace("s", pObj["s"]);
        format = format.replace("a", pObj["a"]);
        return format;
    }
    getParts(epoch) {
        var dateObj = this.getObjFromEpoch(epoch);
        var rObj = {};
        rObj["Y"] = dateObj.getFullYear(); //A full numeric representation of a year, 4 digits
        rObj["n"] = dateObj.getMonth() + 1; //Numeric representation of a month, without leading zeros
        rObj["m"] = dateObj.getMonth() + 1; //Numeric representation of a month, with leading zeros
        if (rObj["m"] < 10) {
            rObj["m"] = "0" + rObj["m"];
        }
        rObj["j"] = dateObj.getDate(); //Day of the month without leading zeros
        rObj["d"] = dateObj.getDate(); //Day of the month, 2 digits with leading zeros
        if (rObj["d"] < 10) {
            rObj["d"] = "0" + rObj["d"];
        }
        rObj["i"] = dateObj.getMinutes(); //Minutes with leading zeros
        if (rObj["i"] < 10) {
            rObj["i"] = "0" + rObj["i"];
        }
        rObj["s"] = dateObj.getSeconds(); //Seconds with leading zeros
        if (rObj["s"] < 10) {
            rObj["s"] = "0" + rObj["s"];
        }
        rObj["H"] = dateObj.getHours(); //24-hour format of an hour with leading zeros
        if (rObj["H"] < 10) {
            rObj["H"] = "0" + rObj["H"];
        }
        rObj["h"] = dateObj.getHours(); //12-hour format of an hour with leading zeros
        if (rObj["h"] > 12) {
            rObj["h"] = rObj["h"] - 12;
        }
        if (rObj["h"] < 10) {
            rObj["h"] = "0" + rObj["h"];
        }
        rObj["a"] = "am";
        if (rObj["H"] > 12) {
            rObj["a"] = "pm";
        }
        rObj["A"] = rObj["a"].toUpperCase();
        rObj["T"] = "UNKNOWN";
        //src: https://stackoverflow.com/questions/1954397/detect-timezone-abbreviation-using-javascript
        try {
            rObj["T"] = /.*\s([A-Z]+)/.exec(dateObj.toLocaleDateString(navigator.language, { timeZoneName: 'short' }))[1];
        }
        catch (e) {
            rObj["T"] = dateObj.toTimeString().match(new RegExp("[A-Z](?!.*[\(])", "g")).join('');
        }
        return rObj;
    }
}

class Time extends BaseFactory {
    constructor() {
        //USE: this.getAuto().get("utilities").getTime().getMicroEpoch();
        super(...arguments);
        this._curTool = null;
        this._formatTool = null;
    }
    getMicroEpoch(asFloat = true) {
        //shortcut alias
        return this.getCurrent().getMicroEpoch(asFloat);
    }
    getCurrent() {
        if (this._curTool === null) {
            this._curTool = new Current();
            this._curTool.setAuto(this.getAuto());
        }
        return this._curTool;
    }
    getFormatter() {
        if (this._formatTool === null) {
            this._formatTool = new Formatter();
            this._formatTool.setAuto(this.getAuto());
        }
        return this._formatTool;
    }
}

class Touch extends BaseTool {
    constructor() {
        super(...arguments);
        this._isSupported = null;
    }
    isSupported() {
        if (this._isSupported === null) {
            //src: https://stackoverflow.com/questions/6262584/how-to-determine-if-the-client-is-a-touch-device
            var el = document.createElement("div");
            el.setAttribute("ongesturestart", "return;"); // or try "ontouchstart"
            try {
                this._isSupported = typeof el["ongesturestart"] === "function";
            }
            catch (e) {
                this._isSupported = false;
            }
            try {
                el.parentNode.removeChild(el);
            }
            catch (e) {
            }
        }
        return this._isSupported;
    }
}

class Clipboard extends BaseTool {
    //var promise	= something.getAuto().get("utilities").getUi().getClipboard().set("myVal", true);
    set(value, doThrow = true) {
        var pObj = this.getAuto().newPromise();
        var pObj2 = this.getAuto().newPromise();
        if (typeof navigator.clipboard !== "undefined") {
            navigator.clipboard.writeText(value).then(() => {
                pObj2["resolve"](true);
            }, (errObj) => {
                pObj2["resolve"](false);
            });
        }
        else {
            pObj2["resolve"](false);
        }
        pObj2.promise.then((bool) => {
            if (bool === false) {
                //fall back, input element
                var el = document.createElement("input");
                el.value = value;
                el.style.top = "0";
                el.style.left = "0";
                el.style.opacity = "0";
                el.style.position = "fixed";
                el.contentEditable = 'true';
                el.readOnly = false;
                document.body.appendChild(el);
                el.select();
                el.focus();
                el.setSelectionRange(0, 999999);
                var res = document.execCommand('copy');
                el.blur();
                document.body.removeChild(el);
                if (res === false) {
                    //fall back, text area
                    var el2 = document.createElement("textarea");
                    el2.value = value;
                    el2.style.top = "0";
                    el2.style.left = "0";
                    el2.style.opacity = "0";
                    el2.style.position = "fixed";
                    document.body.appendChild(el2);
                    el2.focus();
                    el2.select();
                    var res = document.execCommand('copy');
                    document.body.removeChild(el2);
                    if (res === false) {
                        if (doThrow === true) {
                            pObj["reject"](this.getAuto().newException("Failed to set clipboard value"));
                        }
                        else {
                            pObj["resolve"](false);
                        }
                    }
                    else {
                        pObj["resolve"](true);
                    }
                }
                else {
                    pObj["resolve"](true);
                }
            }
            else {
                pObj["resolve"](true);
            }
        });
        return pObj.promise;
    }
    get(doThrow = true) {
        var pObj = this.getAuto().newPromise();
        var pObj2 = this.getAuto().newPromise();
        if (typeof navigator.clipboard !== "undefined") {
            navigator.clipboard.readText().then((value) => {
                pObj2["resolve"](value);
            }, (errObj) => {
                pObj2["resolve"]("mtm-unable-to-copy-content44xx");
            });
        }
        else {
            pObj2["resolve"]("mtm-unable-to-copy-content44xx");
        }
        pObj2.promise.then((value) => {
            if (value === "mtm-unable-to-copy-content44xx") {
                //fall back
                var textArea = document.createElement("textarea");
                textArea.value = value;
                textArea.style.top = "0";
                textArea.style.left = "0";
                textArea.style.opacity = "0";
                textArea.style.position = "fixed";
                document.body.appendChild(textArea);
                textArea.focus();
                textArea.select();
                var res = document.execCommand("paste", null, null);
                value = textArea.value;
                document.body.removeChild(textArea);
                if (res === false) {
                    if (doThrow === true) {
                        pObj["reject"](this.getAuto().newException("Failed to get clipboard value"));
                    }
                    else {
                        pObj["resolve"](false);
                    }
                }
                else {
                    pObj["resolve"](value);
                }
            }
            else {
                pObj["resolve"](value);
            }
        });
        return pObj.promise;
    }
}

class Ui extends BaseFactory {
    constructor() {
        super(...arguments);
        this._touchTool = null;
        this._clipTool = null;
    }
    getTouch() {
        if (this._touchTool === null) {
            this._touchTool = new Touch();
            this._touchTool.setAuto(this.getAuto());
        }
        return this._touchTool;
    }
    getClipboard() {
        if (this._clipTool === null) {
            this._clipTool = new Clipboard();
            this._clipTool.setAuto(this.getAuto());
        }
        return this._clipTool;
    }
}

class Exception extends Error {
    constructor(msg = null, code = null) {
        super(msg);
        //adding auto to exception results in ERROR TypeError: "cyclic object value" when thrown
        this._msg = null;
        this._code = 0;
        Object.setPrototypeOf(this, Exception.prototype);
        if (msg !== null) {
            this._msg = msg;
        }
        if (code !== null) {
            this._code = code;
        }
    }
    getMessage() {
        return this._msg;
    }
    getCode() {
        return this._code;
    }
}

class Exceptions extends BaseFactory {
    get(msg = null, code = null) {
        var rObj = new Exception(msg, code);
        //adding auto to exception results in ERROR TypeError: "cyclic object value" when thrown
        //rObj.setAuto(this.getAuto());
        return rObj;
    }
}

class Cookies extends BaseTool {
    //Use:
    //var bool	= something.getAuto().get("utilities").getStorage().getCookies().getRootDomain();
    //note:A cookie can contains data upto 4096 Bytes only that is the maximum size of a cookie which can be. 
    //If you want to support most browsers, then do not exceed 50 cookies per domain, and 4093 bytes per domain. 
    //That is, the size of all cookies should not exceed 4096 bytes.
    getRootDomain() {
        var host = document.location.hostname;
        var res = host.match(/([^\.]+\.[^\.0-9]+|[^\.]+)$/);
        if (res !== null) {
            return "." + res[0];
        }
        else {
            throw this.getAuto().newException("Cannot determine current root domain");
        }
    }
}

class BaseModel {
    constructor() {
        this._mtmAuto = null;
    }
    setAuto(service) {
        this._mtmAuto = service;
        return this;
    }
    getAuto() {
        return this._mtmAuto;
    }
}

class Cookie extends BaseModel {
    constructor() {
        super(...arguments);
        this._toolObj = null;
        this._name = null; //name
        this._value = null; //data
        this._path = "/"; //path
        this._expires = null; //time to delete, maxAge use setExpiresSecs(secs)
        this._domain = null; //domain access
        this._secure = true; //https only
        this._httpOnly = false; //no JS access
        this._sameSite = "strict";
    }
    getName() {
        return this._name;
    }
    setName(name) {
        this._name = name;
        return this;
    }
    getDomain() {
        return this._domain;
    }
    setDomain(value) {
        this._domain = value;
        return this;
    }
    getValue() {
        return this._value;
    }
    setValue(value) {
        this._value = value;
        return this;
    }
    getValueObj(doThrow = false) {
        try {
            return JSON.parse(atob(this.getValue()));
        }
        catch (e) {
            if (doThrow === true) {
                throw this.getAuto().newException(e);
            }
            else {
                return null;
            }
        }
    }
    setValueObj(obj) {
        this._value = btoa(JSON.stringify(obj));
        return this;
    }
    getPath() {
        return this._path;
    }
    setPath(path) {
        this._path = path;
        return this;
    }
    getSameSite() {
        return this._sameSite;
    }
    setSameSite(value) {
        if (/(strict|lax)/.test(value) === false) {
            throw this.getAuto().newException("Invalid input: " + value);
        }
        this._sameSite = value;
        return this;
    }
    getHttpOnly() {
        return this._httpOnly;
    }
    setHttpOnly(bool) {
        if (bool !== false && bool !== true) {
            throw this.getAuto().newException("Invalid input");
        }
        this._httpOnly = bool;
        return this;
    }
    getSecure() {
        return this._secure;
    }
    setSecure(bool) {
        if (bool !== false && bool !== true) {
            throw this.getAuto().newException("Invalid input");
        }
        this._secure = bool;
        return this;
    }
    setExpiresSecs(secs) {
        this._expires = Math.ceil((this.getAuto().getEpoch() + secs));
        return this;
    }
    setExpiresDays(days) {
        this.setExpiresSecs(days * 24 * 60 * 60);
        return this;
    }
    getExpires() {
        //not available on pull
        //src: https://stackoverflow.com/questions/3274875/how-to-get-cookie-expiration-date-creation-date-from-javascript
        return this._expires;
    }
    push() {
        if (this.getName() === null) {
            throw this.getAuto().newException("Cannot push without a name");
        }
        var str = this.getName();
        str += "=" + this.getValue() + ";";
        if (this.getExpires() !== null) {
            const expires = this.getAuto().get("utilities").getTime().getFormatter().getObjFromEpoch(this.getExpires()).toUTCString();
            str += " expires=" + expires + ";";
        }
        if (this.getPath() !== null) {
            str += " path=" + this.getPath() + ";";
        }
        if (this.getDomain() !== null) {
            str += " domain=" + this.getDomain() + ";";
        }
        if (this.getSameSite() !== null) {
            str += " samesite=" + this.getSameSite() + ";";
        }
        if (this.getSecure() === true) {
            str += " secure;";
        }
        if (this.getHttpOnly() === true) {
            str += " httpOnly;";
        }
        document.cookie = str;
        return this;
    }
    pull(doThrow = false) {
        if (this.getName() === null) {
            throw this.getAuto().newException("Cannot pull without a name");
        }
        else if (this.getExists(doThrow) === false) {
            //cookie does not exist and user does not want to throw
            return this;
        }
        var items = decodeURIComponent(document.cookie).split(';');
        const iLen = items.length;
        for (let i = 0; i < iLen; i++) {
            var str = items[i];
            var index = str.indexOf('=');
            var name = str.slice(0, index).trim();
            if (name === this.getName()) {
                var value = str.slice(index + 1);
                this.setValue(value);
                break;
            }
        }
        return this;
    }
    getExists(doThrow = false) {
        var items = decodeURIComponent(document.cookie).split(';');
        const iLen = items.length;
        for (let i = 0; i < iLen; i++) {
            var str = items[i];
            var index = str.indexOf('=');
            var name = str.slice(0, index).trim();
            if (name === this.getName()) {
                return true;
            }
        }
        if (doThrow === true) {
            throw this.getAuto().newException("Cookie does not exist");
        }
        else {
            return false;
        }
    }
    delete() {
        if (this.getExists() === true) {
            this.setExpiresSecs(-1).push();
        }
    }
    setTool(tool) {
        this._toolObj = tool;
        return this;
    }
    getTool() {
        return this._toolObj;
    }
}

class Storage extends BaseFactory {
    constructor() {
        //Use:
        //var cookieObj	= something.getAuto().get("utilities").getStorage().getCookie("my-cookie");
        super(...arguments);
        this._cookieTool = null;
    }
    getCookie(name = null) {
        var rObj = new Cookie();
        rObj.setAuto(this.getAuto()).setTool(this.getCookieTool());
        if (name !== null) {
            rObj.setName(name);
        }
        return rObj;
    }
    getCookieTool() {
        if (this._cookieTool === null) {
            this._cookieTool = new Cookies();
            this._cookieTool.setAuto(this.getAuto());
        }
        return this._cookieTool;
    }
}

class MtmUtilities {
    constructor() {
        this._mtmAuto = null;
        this._guidsFact = null;
        this._arrFact = null;
        this._strFact = null;
        this._numFact = null;
        this._expFact = null;
        this._objFact = null;
        this._asyncFact = null;
        this._hashFact = null;
        this._netFact = null;
        this._timeFact = null;
        this._uiFact = null;
        this._excepFact = null;
        this._storeFact = null;
    }
    setAuto(mtmAuto) {
        this._mtmAuto = mtmAuto;
        this.getAuto().set("utilities", this);
        return this;
    }
    getAuto() {
        return this._mtmAuto;
    }
    getGuids() {
        if (this._guidsFact === null) {
            this._guidsFact = new Guids();
            this._guidsFact.setAuto(this.getAuto());
        }
        return this._guidsFact;
    }
    getArrays() {
        if (this._arrFact === null) {
            this._arrFact = new Arrays();
            this._arrFact.setAuto(this.getAuto());
        }
        return this._arrFact;
    }
    getStrings() {
        if (this._strFact === null) {
            this._strFact = new Strings$1();
            this._strFact.setAuto(this.getAuto());
        }
        return this._strFact;
    }
    getNumbers() {
        if (this._numFact === null) {
            this._numFact = new Numbers();
            this._numFact.setAuto(this.getAuto());
        }
        return this._numFact;
    }
    getExpressions() {
        if (this._expFact === null) {
            this._expFact = new Expressions();
            this._expFact.setAuto(this.getAuto());
        }
        return this._expFact;
    }
    getObjects() {
        if (this._objFact === null) {
            this._objFact = new Objects$1();
            this._objFact.setAuto(this.getAuto());
        }
        return this._objFact;
    }
    getAsync() {
        if (this._asyncFact === null) {
            this._asyncFact = new Async();
            this._asyncFact.setAuto(this.getAuto());
        }
        return this._asyncFact;
    }
    getHashing() {
        if (this._hashFact === null) {
            this._hashFact = new Hashing();
            this._hashFact.setAuto(this.getAuto());
        }
        return this._hashFact;
    }
    getNetwork() {
        if (this._netFact === null) {
            this._netFact = new Networks();
            this._netFact.setAuto(this.getAuto());
        }
        return this._netFact;
    }
    getTime() {
        if (this._timeFact === null) {
            this._timeFact = new Time();
            this._timeFact.setAuto(this.getAuto());
        }
        return this._timeFact;
    }
    getUi() {
        if (this._uiFact === null) {
            this._uiFact = new Ui();
            this._uiFact.setAuto(this.getAuto());
        }
        return this._uiFact;
    }
    getExceptions() {
        if (this._excepFact === null) {
            this._excepFact = new Exceptions();
            this._excepFact.setAuto(this.getAuto());
        }
        return this._excepFact;
    }
    getStorage() {
        if (this._storeFact === null) {
            this._storeFact = new Storage();
            this._storeFact.setAuto(this.getAuto());
        }
        return this._storeFact;
    }
}
MtmUtilities.ɵprov = ɵɵdefineInjectable({ factory: function MtmUtilities_Factory() { return new MtmUtilities(); }, token: MtmUtilities, providedIn: "root" });
MtmUtilities.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];

/*
 * Public API Surface of mtm-utilities
 */

/**
 * Generated bundle index. Do not edit.
 */

export { MtmUtilities };
//# sourceMappingURL=mtm-utilities.js.map
